class CellCoordinatesException(Exception): ...
class IllegalCharacterError(Exception): ...
class NamedRangeException(Exception): ...
class SheetTitleException(Exception): ...
class InvalidFileException(Exception): ...
class ReadOnlyWorkbookException(Exception): ...
class WorkbookAlreadySaved(Exception): ...
